/**
 * Copyright (c) 2013, 2020 Borland Software Corporation, CEA LIST, Artal and others
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License 2.0
 * which accompanies this distribution, and is available at
 * https://www.eclipse.org/legal/epl-2.0/
 * 
 * SPDX-License-Identifier: EPL-2.0
 * 
 * Contributors:
 *    Michael Golubev (Montages) - initial API and implementation
 *    Aurelien Didier (ARTAL) - aurelien.didier51@gmail.com - Bug 569174
 *    Etienne Allogo (ARTAL) - etienne.allogo@artal.fr - Bug 569174 : L1.2 clean
 */
package metamodel;

import com.google.inject.Inject;
import com.google.inject.Singleton;
import org.eclipse.emf.codegen.ecore.genmodel.GenPackage;
import org.eclipse.emf.common.util.EList;
import org.eclipse.papyrus.gmf.codegen.gmfgen.DynamicModelAccess;
import org.eclipse.xtend2.lib.StringConcatenation;
import org.eclipse.xtext.xbase.lib.Extension;
import xpt.Common;

@Singleton
@SuppressWarnings("all")
public class Facility {
  @Inject
  @Extension
  private Common _common;
  
  @Inject
  @Extension
  private Facility_qvto _facility_qvto;
  
  public CharSequence className(final DynamicModelAccess it) {
    StringConcatenation _builder = new StringConcatenation();
    String _className = it.getClassName();
    _builder.append(_className);
    return _builder;
  }
  
  public CharSequence packageName(final DynamicModelAccess it) {
    StringConcatenation _builder = new StringConcatenation();
    String _packageName = it.getPackageName();
    _builder.append(_packageName);
    return _builder;
  }
  
  public CharSequence qualifiedClassName(final DynamicModelAccess it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(".");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    return _builder;
  }
  
  public CharSequence fullPath(final DynamicModelAccess it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _qualifiedClassName = this.qualifiedClassName(it);
    _builder.append(_qualifiedClassName);
    return _builder;
  }
  
  public CharSequence Main(final DynamicModelAccess it) {
    StringConcatenation _builder = new StringConcatenation();
    _builder.append("package ");
    CharSequence _packageName = this.packageName(it);
    _builder.append(_packageName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("public class ");
    CharSequence _className = this.className(it);
    _builder.append(_className);
    _builder.append(" {");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    {
      EList<GenPackage> _allDomainGenPackages = it.getEditorGen().getAllDomainGenPackages(false);
      for(final GenPackage p : _allDomainGenPackages) {
        _builder.append("\t");
        CharSequence _field = this.field(p, it);
        _builder.append(_field, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("\t");
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private final org.eclipse.emf.ecore.EPackage ePackage;");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_1 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_1, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private ");
    CharSequence _className_1 = this.className(it);
    _builder.append(_className_1, "\t");
    _builder.append("(org.eclipse.emf.ecore.EPackage ePackage) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("this.ePackage = ePackage;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    {
      EList<GenPackage> _allDomainGenPackages_1 = it.getEditorGen().getAllDomainGenPackages(false);
      for(final GenPackage p_1 : _allDomainGenPackages_1) {
        _builder.append("\t");
        CharSequence _method = this.getMethod(p_1, it);
        _builder.append(_method, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.newLine();
    _builder.append("\t");
    CharSequence _registryMethod = this.getRegistryMethod(it);
    _builder.append(_registryMethod, "\t");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_2 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_2, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public boolean isInstance(Object obj, String metaClassName) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EClass metaClass = getEClass(metaClassName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return obj instanceof org.eclipse.emf.ecore.EObject && metaClass.isSuperTypeOf(((org.eclipse.emf.ecore.EObject) obj).eClass());");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_3 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_3, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.emf.ecore.EObject newInstance(String metaClassName) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return this.ePackage.getEFactoryInstance().create(getEClass(metaClassName));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_4 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_4, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.emf.ecore.EClass getEClass(String metaClassName) {");
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence __assert = this._common._assert("metaClassName != null");
    _builder.append(__assert, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EClass metaClass = findMetaClass(metaClassName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("assertMetaClass(metaClass, metaClassName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return metaClass;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_5 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_5, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.emf.ecore.EStructuralFeature getEFeature(String metaClassName, String metaFeatureName) {");
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence __assert_1 = this._common._assert("metaFeatureName != null");
    _builder.append(__assert_1, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EClass metaClass = getEClass(metaClassName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EStructuralFeature f = metaClass.getEStructuralFeature(metaFeatureName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("assertMetaFeature(f, metaClass, metaFeatureName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return f;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_6 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_6, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.emf.ecore.EAttribute getEAttribute(String metaClassName, String metaFeatureName) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return (org.eclipse.emf.ecore.EAttribute) getEFeature(metaClassName, metaFeatureName);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_7 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_7, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.emf.ecore.EReference getEReference(String metaClassName, String metaFeatureName) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return (org.eclipse.emf.ecore.EReference) getEFeature(metaClassName, metaFeatureName);");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_8 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_8, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.emf.ecore.EDataType getEDataType(String metaClassName) {");
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence __assert_2 = this._common._assert("metaClassName != null");
    _builder.append(__assert_2, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EClassifier c = this.ePackage.getEClassifier(metaClassName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (false == c instanceof org.eclipse.emf.ecore.EDataType) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return (org.eclipse.emf.ecore.EDataType) c;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_9 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_9, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("public org.eclipse.emf.ecore.EEnum getEEnum(String metaClassName) {");
    _builder.newLine();
    _builder.append("\t\t");
    CharSequence __assert_3 = this._common._assert("metaClassName != null");
    _builder.append(__assert_3, "\t\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EClassifier c = this.ePackage.getEClassifier(metaClassName);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (false == c instanceof org.eclipse.emf.ecore.EEnum) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return (org.eclipse.emf.ecore.EEnum) c;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_10 = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment_10, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private org.eclipse.emf.ecore.EClass findMetaClass(String name) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EClassifier c = this.ePackage.getEClassifier(name);");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (false == c instanceof org.eclipse.emf.ecore.EClass) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("return null;");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("return (org.eclipse.emf.ecore.EClass) c;");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    _builder.append("\t");
    CharSequence _generatedMemberComment_11 = this._common.generatedMemberComment("Default implementation throws IllegalStateException if metaclass is null");
    _builder.append(_generatedMemberComment_11, "\t");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("private void assertMetaClass(org.eclipse.emf.ecore.EClass metaClass, String metaClassName) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("if (metaClass == null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("throw new IllegalStateException(java.text.MessageFormat.format(\"Can\'t find class {0} in the package {1}\", new Object[] {metaClassName, this.ePackage.getName()}));");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.newLine();
    CharSequence _generatedMemberComment_12 = this._common.generatedMemberComment("Default implementation throws IllegalStateException if meta-feature is null");
    _builder.append(_generatedMemberComment_12);
    _builder.newLineIfNotEmpty();
    _builder.append("private void assertMetaFeature(org.eclipse.emf.ecore.EStructuralFeature metaFeature, org.eclipse.emf.ecore.EClass metaClass, String featureName) {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("if (metaFeature == null) {");
    _builder.newLine();
    _builder.append("\t\t");
    _builder.append("throw new IllegalStateException(java.text.MessageFormat.format(\"Can\'t find feature {0} of class {1} in the package {2}\", new Object[] {featureName, metaClass.getName(), this.ePackage.getName()}));");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    {
      EList<GenPackage> _allDomainGenPackages_2 = it.getEditorGen().getAllDomainGenPackages(false);
      for(final GenPackage p_2 : _allDomainGenPackages_2) {
        _builder.append("\t");
        CharSequence _initMethod = this.initMethod(p_2);
        _builder.append(_initMethod, "\t");
        _builder.newLineIfNotEmpty();
      }
    }
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence field(final GenPackage it, final DynamicModelAccess dma) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private static ");
    CharSequence _className = this.className(dma);
    _builder.append(_className);
    _builder.append(" ");
    String _fieldName = this._facility_qvto.fieldName(it);
    _builder.append(_fieldName);
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    return _builder;
  }
  
  public CharSequence getMethod(final GenPackage it, final DynamicModelAccess dma) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment("@throws IllegalStateException if no EPackage with given URI is registered.");
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("public static ");
    CharSequence _className = this.className(dma);
    _builder.append(_className);
    _builder.append(" get");
    String _nameToken = this._facility_qvto.getNameToken(it);
    _builder.append(_nameToken);
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("if (");
    String _fieldName = this._facility_qvto.fieldName(it);
    _builder.append(_fieldName, "\t");
    _builder.append(" == null) {");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("org.eclipse.emf.ecore.EPackage pkg = getRegistry().getEPackage(\"");
    String _nsURI = it.getEcorePackage().getNsURI();
    _builder.append(_nsURI, "\t\t");
    _builder.append("\");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("if (pkg == null) {");
    _builder.newLine();
    _builder.append("\t\t\t");
    _builder.append("throw new IllegalStateException(\"Package ");
    String _name = it.getEcorePackage().getName();
    _builder.append(_name, "\t\t\t");
    _builder.append("(");
    String _nsURI_1 = it.getEcorePackage().getNsURI();
    _builder.append(_nsURI_1, "\t\t\t");
    _builder.append(") not found\");");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t\t");
    String _fieldName_1 = this._facility_qvto.fieldName(it);
    _builder.append(_fieldName_1, "\t\t");
    _builder.append(" = new ");
    CharSequence _className_1 = this.className(dma);
    _builder.append(_className_1, "\t\t");
    _builder.append("(pkg);");
    _builder.newLineIfNotEmpty();
    _builder.append("\t\t");
    String _fieldName_2 = this._facility_qvto.fieldName(it);
    _builder.append(_fieldName_2, "\t\t");
    _builder.append(".init");
    String _nameToken_1 = this._facility_qvto.getNameToken(it);
    _builder.append(_nameToken_1, "\t\t");
    _builder.append("();");
    _builder.newLineIfNotEmpty();
    _builder.append("\t");
    _builder.append("}");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return ");
    String _fieldName_3 = this._facility_qvto.fieldName(it);
    _builder.append(_fieldName_3, "\t");
    _builder.append(";");
    _builder.newLineIfNotEmpty();
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence initMethod(final GenPackage it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment();
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private void init");
    String _nameToken = this._facility_qvto.getNameToken(it);
    _builder.append(_nameToken);
    _builder.append("() {");
    _builder.newLineIfNotEmpty();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
  
  public CharSequence getRegistryMethod(final DynamicModelAccess it) {
    StringConcatenation _builder = new StringConcatenation();
    CharSequence _generatedMemberComment = this._common.generatedMemberComment("Default implementation returns global registry, clients that need another may redefine.");
    _builder.append(_generatedMemberComment);
    _builder.newLineIfNotEmpty();
    _builder.append("private static org.eclipse.emf.ecore.EPackage.Registry getRegistry() {");
    _builder.newLine();
    _builder.append("\t");
    _builder.append("return org.eclipse.emf.ecore.EPackage.Registry.INSTANCE;");
    _builder.newLine();
    _builder.append("}");
    _builder.newLine();
    return _builder;
  }
}
